#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qiconview.h>
#include <qlabel.h>
#include <qlayout.h>

#include "diskview.h"

#include "deviceview.h"
#include "deviceview.moc"

#undef ENABLE_EDITS

#define i18n(x) x

static QString cmdText( "%1" );

DeviceView::DeviceView( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  QVBoxLayout *box = new QVBoxLayout( this, 6, 6 );

  //
  // Partition List
  //
  disk = new DiskView( this );
  box->addWidget( disk );
  connect( disk, SIGNAL( partitionSelected( PedPartition * ) ),
	   this, SLOT( setPartition( PedPartition * ) ) );

  //
  // Partition Details
  //
  QGroupBox *partDetails = new QGroupBox( 3, Horizontal, i18n("Details"), this );
  bootablePart = new QCheckBox( i18n( "Bootable" ), partDetails );
  rootPart = new QCheckBox( i18n( "Root" ), partDetails );
  swapPart = new QCheckBox( i18n( "Swap" ), partDetails );
  hiddenPart = new QCheckBox( i18n( "Hidden" ), partDetails );
  raidPart = new QCheckBox( i18n( "RAID" ), partDetails );
  lvmPart = new QCheckBox( i18n( "LVM" ), partDetails );
  lbaPart = new QCheckBox( i18n( "LBA" ), partDetails );
  setPartition( 0 );

  box->addWidget( partDetails );

#ifdef ENABLE_EDITS
  //
  // Command Area
  //
  cmdLabel = new QLabel( cmdText.arg( i18n("<qt>This tool allows you to examine and modify "
					   "a hard disk's partition table. <b>Using this program "
					   "incorrecly can damage your data, computer etc. "
					   "beyond all hope of recovery.</b></qt>") ),
			 this );
  cmdLabel->setFrameStyle( QFrame::Box | QFrame::Plain );
  cmdLabel->setLineWidth( 2 );
  cmdLabel->setBackgroundColor( QColor( "lightYellow" ) );
  box->addWidget( cmdLabel );

  cmds = new QIconView( this );
  cmds->setItemTextPos( QIconView::Right );
  cmds->setArrangement( QIconView::LeftToRight );
  box->addWidget( cmds );

  new QIconViewItem( cmds, i18n("Create") );
  new QIconViewItem( cmds, i18n("Destroy") );
  new QIconViewItem( cmds, i18n("Resize") );
  new QIconViewItem( cmds, i18n("Move") );
  new QIconViewItem( cmds, i18n("Make Image") );
  cmds->arrangeItemsInGrid();

#endif
}

DeviceView::~DeviceView()
{
}

void DeviceView::resizeEvent( QResizeEvent *e )
{
  QWidget::resizeEvent( e );
#ifdef ENABLE_EDITS
  cmds->arrangeItemsInGrid();
#endif
}

void DeviceView::setDevice( PedDevice *dev )
{
  PedDisk *dsk = ped_disk_open( dev );
  if ( !dsk ) {
    fprintf(stderr,  "Could not open disk\n" );
    exit(1);
  }

  setPartition( 0 );
  emit setTitle( dev->path, dev->model );
  disk->setDisk( dsk );
}

void DeviceView::setPartition( PedPartition *part )
{
  if ( !part ) {
    bootablePart->setChecked( false );
    rootPart->setChecked( false );
    swapPart->setChecked( false );
    hiddenPart->setChecked( false );
    raidPart->setChecked( false );
    lvmPart->setChecked( false );
    lbaPart->setChecked( false );

    bootablePart->setEnabled( false );
    rootPart->setEnabled( false );
    swapPart->setEnabled( false );
    hiddenPart->setEnabled( false );
    raidPart->setEnabled( false );
    lvmPart->setEnabled( false );
    lbaPart->setEnabled( false );

    return;
  }

  updateFlag( bootablePart, part, PED_PARTITION_BOOT );
  updateFlag( rootPart, part, PED_PARTITION_ROOT );
  updateFlag( swapPart, part, PED_PARTITION_SWAP );
  updateFlag( hiddenPart, part, PED_PARTITION_HIDDEN );
  updateFlag( raidPart, part, PED_PARTITION_RAID );
  updateFlag( lvmPart, part, PED_PARTITION_LVM );
  updateFlag( lbaPart, part, PED_PARTITION_LBA );
}

void DeviceView::updateFlag( QCheckBox *check, PedPartition *part, PedPartitionFlag flag )
{
  if ( ped_partition_is_flag_available( part, flag ) ) {
    check->setEnabled( true );
    check->setChecked( ped_partition_get_flag( part, flag ) ? true : false );
  }
  else {
    check->setChecked( false );
    check->setEnabled( false );
  }

}
